#include <AppKit.h>
#include <InterfaceKit.h>
#include <stdio.h>
//ThreadPrimitive is discussed here:
//http://www.beatjapan.org/mirror/www.be.com/aboutbe/benewsletter/volume_II/Issue32.html#Insight
#include "ThreadPrimitive.h"

//The Overlay code is based-on/inspired by the method used in VLC here:
//http://www.videolan.org/cgi-bin/cvsweb/vlc/plugins/beos/Attic/vout_beos.cpp?rev=1.24

class OverlayWindow : public BWindow,ThreadPrimitive {

	private:
		BView * OverlayView;
		BStringView * FPS;
		BBitmap * OverlayBitmap[2] ;
		int active;
		volatile bool alive;
	public:
		OverlayWindow() : BWindow(BRect(50,50,370,290),"Overlay Test",B_TITLED_WINDOW,0) {
			//setup the Ovelay's BView
			this->AddChild(OverlayView = new BView(BRect(0,0,319,239),"Overlay View",B_FOLLOW_ALL_SIDES,B_WILL_DRAW));
			#ifdef OVERLAY
				OverlayBitmap[0] = new BBitmap(BRect(0,0,319,239),B_BITMAP_WILL_OVERLAY|B_BITMAP_RESERVE_OVERLAY_CHANNEL,B_RGB32);
				OverlayBitmap[1] = new BBitmap(BRect(0,0,319,239),B_BITMAP_WILL_OVERLAY,B_RGB32);

				//it helps to get the overlay color before the window is drawn to set the view color correctly
				rgb_color OverlayColor;
				OverlayView->SetViewOverlay(OverlayBitmap[0],&OverlayColor,B_FOLLOW_ALL_SIDES,B_OVERLAY_FILTER_HORIZONTAL|B_OVERLAY_FILTER_VERTICAL);
				OverlayView->SetViewColor(OverlayColor);
			#else
				OverlayBitmap[0] = new BBitmap(BRect(0,0,319,239),B_RGB32);
				OverlayBitmap[1] = new BBitmap(BRect(0,0,319,239),B_RGB32);
			#endif

			active=0; //set current frame in the doouble-buffer setup
			
			//FPS Display
			OverlayView->AddChild(FPS=new BStringView(BRect(0,0,300,48),"FPS","FPS"));
			FPS->SetFontSize(32);
			FPS->SetHighColor((rgb_color){255,0,0,0});

			//start the rendering thread
			alive=true;
			this->Go();
		}
		
		void RunThread() {
			float s=1.0;
			bool direction=false;
			int ticks = 0;
			bigtime_t watch[2];
			watch[0]=system_time();
			float timer=0;
			char FPSString[16];
			while (alive) {

				//build bitmap (taking advantage of 4-way symmetry
				rgb_color * Bitmap = (rgb_color*)OverlayBitmap[active=(active+1)%2]->Bits();
				int bpr=OverlayBitmap[active]->BytesPerRow()/4; //precompute this, since we'll use it a bunch
				for (int x =0;x<160;x++) {
					for (int y=0;y<120;y++) {
						float d;
						d = sqrt((x-160)*(x-160)+(y-120)*(y-120))*s;
						d=sin(d);
						char b = (int)(127*d+127);
						Bitmap[y*bpr+x]=(rgb_color){b,b,b,0};
						Bitmap[(239-y)*bpr+x]=(rgb_color){b,b,b,0};
						Bitmap[y*bpr+(319-x)]=(rgb_color){b,b,b,0};
						Bitmap[(239-y)*bpr+(319-x)]=(rgb_color){b,b,b,0};
					}
				}
				
				//adjust scaling based on the direction
				if (direction) {
					s/=1.002;
				} else {
					s*=1.002;
				}
				
				//switch scaling when limits hit
				if (s<0.5) direction=false;
				if (s>30) direction=true;

				//Send to screen
				if (LockWithTimeout(500)==B_OK) { //Window must be locked for theese operations
				
					//Update FPS display every 25 frames
					ticks=(ticks+1)%25;
					if (!ticks) {
						watch[1]=watch[0]; //shift stopwatch values
						watch[0]=system_time();
						timer=(float)(watch[0]-watch[1]);
						timer/=1000.0*1000.0;
						snprintf((char *)&FPSString,15,"%4.3f",25.0/timer);
						FPS->SetText((char *)&FPSString);
					}
					
					//Update View
					BScreen * scr = new BScreen(this);
					scr->WaitForRetrace(1000000); //try to sync with the refresh if the card supports it
					#if OVERLAY
						rgb_color OverlayColor;
						OverlayView->SetViewOverlay(OverlayBitmap[active],BRect(0,0,319,239),Bounds(),&OverlayColor,B_FOLLOW_ALL_SIDES,B_OVERLAY_FILTER_HORIZONTAL|B_OVERLAY_FILTER_VERTICAL|B_OVERLAY_TRANSFER_CHANNEL);
						OverlayView->SetViewColor(OverlayColor);
					#else
						OverlayView->DrawBitmap(OverlayBitmap[active], BRect(0,0,319,239), Bounds());
					#endif
					delete scr;
					Unlock();
				}
			}
		}

		bool QuitRequested() {
			be_app->PostMessage(B_QUIT_REQUESTED);
			return true;
		}

		~OverlayWindow() {
			//wait for rendering thread to die before deleting window
			alive=false;
			wait_for_thread(scanThread,NULL);
		}

};


int main() {

	//make application & show the window
	new BApplication("application/x-vnd.OverlayTest");
	(new OverlayWindow)->Show();
	be_app->Run();
	delete be_app;
	
	return 0;

}

